module Olympic
  module Match

    extend ActiveSupport::Concern

    included do
      belongs_to :tournament,
        class_name:   Olympic::Settings[:tournament_class],
        foreign_key:  Olympic::Settings[:match_tournament_key],
        primary_key:  Olympic::Settings[:match_primary_key]
      # An outgoing source is a source that this match is a part of.
      # There are many of these, unfortunately.
      has_many :outgoing,
        class_name:   Olympic::Settings[:source_class],
        foreign_key:  Olympic::Settings[:source_source_key],
        foreign_type: Olympic::Settings[:source_source_type],
        primary_key:  Olympic::Settings[:match_primary_key],
        as: :sources
      # An incoming source is a source that this match is owning.
      # These are the sources that will participate in the match.
      has_many :incoming,
        class_name:   Olympic::Settings[:source_class],
        foreign_key:  Olympic::Settings[:source_match_key],
        primary_key:  Olympic::Settings[:match_primary_key]
      belongs_to :winner,
        class_name:   Olympic::Settings[:team_class],
        foreign_key:  Olympic::Settings[:match_winner_key],
        primary_key:  Olympic::Settings[:team_primary_key]
    end

    # If this match is ready to commence.  This means that the
    # participants are defined.
    #
    # @return [Boolean]
    def ready?
      !!participants
    end

    # If this match has a winner defined; or, in other words, is
    # completed.
    #
    # @return [Boolean]
    def completed?
      winner?
    end

    # Returns the teams that are a part of the match.  If the teams
    # cannot be decided, it will return nil.
    #
    # @return [Array<Olympic::Team>, nil]
    def participants
      winners = incoming.includes(:source).map do |income|
        source = income.source
        case source
        when Settings.class_for(:team)
          source
        when Settings.class_for(:match)
          source.winner
        else
          raise Olympic::Error, "Unknown source #{source}"
        end
      end

      if winners.any? { |winner| winner == nil }
        nil
      else
        winners
      end
    end
  end
end
