require "memoist"
require "forwardable"
require "olympic/rating/glicko/formula"

module Olympic

  # note:
  #   Match#rating
  #   Match#outcome
  #   Match#derivation
  #
  module Rating
    # The Glicko rating system.  This is Glicko, not Glicko2.
    #
    # @see http://www.glicko.net/glicko/glicko.pdf
    class Glicko < Base

      extend Memoist
      extend Forwardable
      def_delegators :@team, :rating=, :derivation=

      # (see Base.required_fields)
      def self.required_fields
        {
          rating:     [:float, { null: false,
                                 default: Formula::DEFAULT_RATING }],
          derivation: [:float, { null: false,
                                 default: Formula::DEFAULT_RATING_DERIVATION }]
        }
      end

      def initialize(team)
        @team = team
      end

      # Updates the rating with the given match information.  Matches
      # should be an array of hashes that contain information about
      # those matches.
      #
      # @param matches [Array<Olympic::Match>]
      # @return [void]
      def update(matches)
        standardized = standardize_matches(matches)
        Formula.new(self, standardized).call
      end

      def rating
        @team.rating ||= 1500
      end

      def derivation
        case
        when unrated?         then 350
        when time_passed == 0 then @team.derivation
        else
          [
            30,
            Math.sqrt(@team.derivation ** 2 +
              CERTAINTY_DECAY * time_passed),
            350
          ].sort[1]
        end
      end
      memoize :derivation

      def time_passed
        # TODO
        0
      end

      def unrated?
        @team.rating == nil && @team.derivation == nil
      end

      private

      # @param matches [Array<Olympic::Match>]
      # @return [Array<Hash>]
      def standardize_matches(matches)
        matches.map do |match|
          raise Rating::Error, "Too many/few incoming sources" unless
            match.incoming.size == 2
          other = match.participants.find { |team| team != @team }
          raise Rating::Error, "Could not find combatant" unless other
          other = Glicko.new(other)
          {
            rating: other.rating,
            derivation: other.derivation,
            outcome: if match.winner == @team then 1 else 0 end
          }
        end
      end
    end
  end
end
