require 'forwardable'
require 'olympic/settings/class_methods'

module Olympic
  class Settings

    extend ClassMethods
    extend Forwardable

    def_delegators :@settings, :fetch

    DEFAULT_SETTINGS = {
      match_class: "Match",
      match_primary_key: :id,
      match_tournament_key: :tournament_id,
      match_foreign_key: :match_id,
      match_winner_key: :winner_id,
      match_join_key: :match_id,
      team_class: "Team",
      team_primary_key:  :id,
      team_tournament_join: :teams_tournaments,
      team_join_key: :team_id,
      tournament_class: "Tournament",
      tournament_primary_key: :id,
      tournament_root_key: :root_id,
      tournament_join_key: :tournament_id,
      source_class: "Source",
      source_primary_key: :id,
      source_match_key: :match_id,
      source_source_key: :source_id,
      source_source_type: :source_type,
      rating: :glicko
    }

    def initialize(settings = {})
      @settings = DEFAULT_SETTINGS.clone.merge(settings)
      @klass = {}
    end

    def build
      yield self
      self
    end

    def []=(key, value)
      @klass = {}
      @settings[key.to_s.to_sym] = value
    end

    def [](key)
      @settings.fetch(key.to_s.to_sym)
    end

    def class_for(name)
      @klass.fetch(name) do
        @klass[name] = fetch(:"#{name}_class").constantize
      end
    end

    def rating_system
      Rating.for(rating)
    end

    def method_missing(method, *args, &block)
      return super if args.length > 1 || block_given?

      method = method.to_s
      case method
      when /\?\z/, /\!\z/
        super
      when /\A(.*)\=\z/
        super unless args.length == 1
        self[$+] = args[0]
      else
        super unless args.length == 0
        self[method]
      end
    end
  end
end
